@@ -1,21 +1,8 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * blkid probe utilities
  *
  * Copyright (C) 2018-2024 Red Hat, Inc. All rights reserved.
- *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #include <errno.h>
@@ -183,7 +170,7 @@ int blk_is_partition(struct blkid_handle *h)
 
 int blk_is_superblock(struct blkid_handle *h)
 {
-	return blkid_probe_has_value(h->pr, "TYPE");;
+	return blkid_probe_has_value(h->pr, "TYPE");
 }
 
 const char *blk_get_partition_type(struct blkid_handle *h)
