@@ -1,22 +1,9 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * Linux block devices helpers
  *
- * Copyright (C) 2018-2023 Red Hat, Inc. All rights reserved.
- * Copyright (C) 2018-2023 Ondrej Kozina
- *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ * Copyright (C) 2018-2024 Red Hat, Inc. All rights reserved.
+ * Copyright (C) 2018-2024 Ondrej Kozina
  */
 
 #include "cryptsetup.h"
@@ -222,17 +209,22 @@ int tools_detect_signatures(const char *device, tools_probe_filter_info filter,
 
 	switch (filter) {
 	case PRB_FILTER_LUKS:
+		log_dbg("Blkid check (filter LUKS).");
 		if (blk_superblocks_filter_luks(h)) {
 			r = -EINVAL;
+			log_dbg("Blkid filter LUKS probe failed.");
 			goto out;
 		}
 		/* fall-through */
 	case PRB_FILTER_NONE:
+		log_dbg("Blkid check (filter none).");
 		blk_set_chains_for_full_print(h);
 		break;
 	case PRB_ONLY_LUKS:
+		log_dbg("Blkid check (LUKS only).");
 		blk_set_chains_for_fast_detection(h);
 		if (blk_superblocks_only_luks(h)) {
+			log_dbg("Blkid only LUKS probe failed.");
 			r = -EINVAL;
 			goto out;
 		}
@@ -251,8 +243,11 @@ int tools_detect_signatures(const char *device, tools_probe_filter_info filter,
 		(*count)++;
 	}
 
-	if (pr == PRB_FAIL)
-		r = -EINVAL;
+	if (pr == PRB_FAIL) {
+		/* Expect device cannot be read */
+		r = -EIO;
+		log_dbg("Blkid probe failed.");
+	}
 out:
 	blk_free(h);
 	return r;
@@ -302,6 +297,8 @@ int tools_wipe_all_signatures(const char *path, bool exclusive, bool only_luks)
 		goto out;
 	}
 
+	log_dbg("Blkid wipe.");
+
 	while ((pr = blk_probe(h)) < PRB_EMPTY) {
 		if (blk_is_partition(h))
 			log_verbose(_("Existing '%s' partition signature on device %s will be wiped."),
