@@ -1,21 +1,8 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * simple unit test for utils_io.c (blockwise low level functions)
  *
- * Copyright (C) 2018-2020 Red Hat, Inc. All rights reserved.
- *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ * Copyright (C) 2018-2024 Red Hat, Inc. All rights reserved.
  */
 
 #include <errno.h>
@@ -99,7 +86,7 @@ static int test_write_buffer(void)
 	if (ret < 0)
 		goto out;
 
-	return (size_t) ret == test_length ? 0 : -EIO;
+	ret = (size_t) ret == test_length ? 0 : -EIO;
 out:
 	if (fd >= 0)
 		close(fd);
@@ -311,6 +298,12 @@ int main(int argc, char **argv)
 	long ps;
 	int r = EXIT_FAILURE;
 
+#ifndef NO_CRYPTSETUP_PATH
+	if (getenv("CRYPTSETUP_PATH")) {
+		printf("Cannot run this test with CRYPTSETUP_PATH set.\n");
+		exit(77);
+	}
+#endif
 	if (parse_input_params(argc, argv))
 		return r;
 
