@@ -1,28 +1,15 @@
+// SPDX-License-Identifier: LGPL-2.1-or-later
 /*
  * Example of LUKS2 ssh token handler (EXPERIMENTAL)
  *
- * Copyright (C) 2016-2021 Milan Broz <gmazyland@gmail.com>
- * Copyright (C) 2020-2021 Vojtech Trefny
+ * Copyright (C) 2016-2024 Milan Broz
+ * Copyright (C) 2020-2024 Vojtech Trefny
  *
  * Use:
  *  - generate LUKS device
  *  - store passphrase used in previous step remotely (single line w/o \r\n)
  *  - add new token using this example
  *  - activate device by token
- *
- * This file is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- *
- * This file is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this file; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #include <stdio.h>
@@ -52,13 +39,20 @@ int cryptsetup_token_open(struct crypt_device *cd, int token,
 	char **password, size_t *password_len, void *usrptr);
 void cryptsetup_token_dump(struct crypt_device *cd, const char *json);
 int cryptsetup_token_validate(struct crypt_device *cd, const char *json);
-
+void cryptsetup_token_buffer_free(void *buffer, size_t buffer_len);
 
 const char *cryptsetup_token_version(void)
 {
 	return TOKEN_VERSION_MAJOR "." TOKEN_VERSION_MINOR;
 }
 
+void cryptsetup_token_buffer_free(void *buffer, size_t buffer_len)
+{
+	/* libcryptsetup API call */
+	crypt_safe_memzero(buffer, buffer_len);
+	free(buffer);
+}
+
 static json_object *get_token_jobj(struct crypt_device *cd, int token)
 {
 	const char *json_slot;
@@ -80,6 +74,9 @@ int cryptsetup_token_open_pin(struct crypt_device *cd, int token, const char *pi
 	ssh_session ssh;
 
 	jobj_token = get_token_jobj(cd, token);
+	if (!jobj_token)
+		return -ENOMEM;
+
 	json_object_object_get_ex(jobj_token, "ssh_server", &jobj_server);
 	json_object_object_get_ex(jobj_token, "ssh_user",   &jobj_user);
 	json_object_object_get_ex(jobj_token, "ssh_path",   &jobj_path);
@@ -87,6 +84,7 @@ int cryptsetup_token_open_pin(struct crypt_device *cd, int token, const char *pi
 
 	r = ssh_pki_import_privkey_file(json_object_get_string(jobj_keypath), pin, NULL, NULL, &pkey);
 	if (r != SSH_OK) {
+		json_object_put(jobj_token);
 		if (r == SSH_EOF) {
 			crypt_log(cd, CRYPT_LOG_ERROR, "Failed to open and import private key.\n");
 			return -EINVAL;
@@ -98,6 +96,7 @@ int cryptsetup_token_open_pin(struct crypt_device *cd, int token, const char *pi
 	ssh = sshplugin_session_init(cd, json_object_get_string(jobj_server),
 				   json_object_get_string(jobj_user));
 	if (!ssh) {
+		json_object_put(jobj_token);
 		ssh_key_free(pkey);
 		return -EINVAL;
 	}
@@ -111,6 +110,7 @@ int cryptsetup_token_open_pin(struct crypt_device *cd, int token, const char *pi
 
 	ssh_disconnect(ssh);
 	ssh_free(ssh);
+	json_object_put(jobj_token);
 
 	return r ? -EINVAL : r;
 }
@@ -135,14 +135,14 @@ void cryptsetup_token_dump(struct crypt_device *cd, const char *json)
 	json_object_object_get_ex(jobj_token, "ssh_path",   &jobj_path);
 	json_object_object_get_ex(jobj_token, "ssh_keypath",&jobj_keypath);
 
-	snprintf(buf, sizeof(buf) - 1, "\tssh_server: %s\n\tssh_user: %s\n"
-		"\tssh_path: %s\n\tssh_key_path: %s\n",
-		json_object_get_string(jobj_server),
-		json_object_get_string(jobj_user),
-		json_object_get_string(jobj_path),
-		json_object_get_string(jobj_keypath));
+	if (snprintf(buf, sizeof(buf) - 1, "\tssh_server: %s\n\tssh_user: %s\n"
+	    "\tssh_path: %s\n\tssh_key_path: %s\n",
+	    json_object_get_string(jobj_server),
+	    json_object_get_string(jobj_user),
+	    json_object_get_string(jobj_path),
+	    json_object_get_string(jobj_keypath)) > 0)
+		crypt_log(cd, CRYPT_LOG_NORMAL, buf);
 
-	crypt_log(cd, CRYPT_LOG_NORMAL, buf);
 	json_object_put(jobj_token);
 }
 
